
;---------------------------------------------------------------------------
; macro COMPWAVETABLE1 
;
;	call this once before a series of COMPWAVETABLE calls
;	(p)reserve PSVPAG,XMODSRT,XMODEND, MODCON, W1...
;
; arguments: 
;	- PHASORSTEP: 32bit (linear freq) pitch
;	- TABLE : 256-word PSV (ROM) wavetable
;	
;	execution time : 9 cycles
;---------------------------------------------------------------------------
.macro PREPCOMPWAVETABLE1 TABLE
		mov 	#psvpage(&TABLE&),W0
		mov 	W0,PSVPAG
		mov		#psvoffset(&TABLE&),W0
		mov		W0,XMODSRT
		mov		#psvoffset(&TABLE&)+#0x1FF,W0
		mov		W0,XMODEND
		mov		#0x8008,W0; W8 is circular adressing ptr
		mov		W0,MODCON
		mov		#0x7FFF,W1
.endm

;---------------------------------------------------------------------------
; macro COMPWAVETABLE1 
;
;	computes one sample of a wavetable synth
; 	linear interpolated 256pt wavetable
;
; 	todo: array in/out operation?
;
; 	phasor (and phasorstep) format:
;   --------MSB-------- --------LSB--------
;   xxxx xxxi iiii iiif ffff ffff ffff ffxx
; 	(i = index in table, f = interpolated fraction)
;
; arguments: 
;	- PHASORSTEP (r): 32bit (linear freq) pitch
;	- TABLE (ro): 256-word PSV (ROM) wavetable
;	- PHASOR (rw): 32bit (linear freq) pitch (keep private)
;	- OUTPUT (w): 16bit wave
;
;	context:
; 	- destroys WREGs
;	
;	execution time : 
;---------------------------------------------------------------------------
.macro COMPWAVETABLE1 PHASORSTEP TABLE PHASOR OUTPUT
;		bclr	update,#1
; Step the phasor
		mov		&PHASORSTEP&,W0
		add	 	&PHASOR&
		mov	 	&PHASORSTEP&+2,W0
		addc 	&PHASOR&+2
; linear interpolation...
		mov 	&PHASOR&+2,W8
		and 	#0x01FE,W8
		mov		#psvoffset(&TABLE&),W0
		add		W8,W0,W8 
		mov	 	[W8++], W4
;		add	 	W8,#2,W8
		lsr		&PHASOR&+2,WREG
		rrc		&PHASOR&,WREG
		lsr		W0,#1,W5
		xor	 	W5,W1,W6		
		mpy		W4*W6,A,[W8],W4
		mac		W4*W5,A
		sac.r 	A, #0, W0
		mov		W0,&OUTPUT&
.endm



;---------------------------------------------------------------------------
; macro MTOF
;
; Midi to frequency 
; actually not frequency but wavetable pointer step
; actually not midi but 0xoffg where o = octave, ff = fraction of octave, g = unused
;
; precision: NON-interpolated 256 steps per octave table
;
;	arguments:
;	- PITCH (r) : unsigned 16 bit word 
;	- PHASORSTEP (w) : unsigned 32 bit word 
;
;	context:
; 	- destroys WREGs
;	
;	execution time : 
;---------------------------------------------------------------------------
.macro MTOF PITCH PHASORSTEP
		mov 	#psvpage(ExpTable),W0
		mov 	W0,PSVPAG
		mov		#0x0FF0,W0
		and		&PITCH&,WREG
		lsr		W0,#3,W0
		mov		#psvoffset(ExpTable),W1
		add		W0,W1,W0
		mov		[W0],W1
		clr 	A
		mov		W1,ACCAL
		mov		&PITCH&,WREG
		lsr		W0,#12,W0
		subr	W0,#9,W0
		sftac	A,W0
		mov		ACCAH,W0
		mov		W0,&PHASORSTEP&+2
		mov		ACCAL,W0
		mov 	W0,	&PHASORSTEP&
.endm

;---------------------------------------------------------------------------
; macro MTOF2 
;
; Midi to frequency 
; actually not frequency but wavetable pointer step
; actually not midi but 0xoffg where o = octave, ff = fraction of octave, g = interpolated fraction
;
; precision: linear interpolated 256 steps per octave table
;
;	arguments:
;	- PITCH (r) : unsigned 16 bit word 
;	- PHASORSTEP (w) : unsigned 32 bit word 
;
;	context:
; 	- destroys WREGs
;	
;	execution time : 22 cycles
;---------------------------------------------------------------------------
.macro MTOF2 PITCH PHASORSTEP
		mov 	#psvpage(ExpTable),W0
		mov 	W0,PSVPAG
		mov		&PITCH&,W1
		mov		#0x0FF0,W0
		and		W0,W1,W0
		lsr		W0,#3,W0
		mov		#psvoffset(ExpTable),W8
		add		W0,W8,W8
		mov		[W8],W4
		add	 	W8,#2,W8
		mov		#0x0F,W0
		and		W0,W1,W5
		xor	 	W5,#0x0F,W6				
		mpy		W4*W6,A,[W8],W4
		mac		W4*W5,A
		lsr		W1,#12,W0
		subr	W0,#9,W0
		sftac	A,W0
		mov		ACCAH,W0
		mov		W0,&PHASORSTEP&+2
		mov		ACCAL,W0
		mov 	W0,	&PHASORSTEP&
.endm


;---------------------------------------------------------------------------
; macro RNG16 seed
;
;	16 bit LFSR pseudo-random generator
;
;	arguments:
;	- seed (rw) : 16 bit seed input and new value output
;
;	context:
; 	- destroys WREGs
;---------------------------------------------------------------------------
.macro	RNG16 SEED
		bclr	SR,#C
		btss	&SEED&,#15
		btg		SR,#C
		btss	&SEED&,#14
		btg		SR,#C
		btss	&SEED&,#12
		btg		SR,#C
		btss	&SEED&,#3
		btg		SR,#C
		rlc		&SEED&
.endm
