
#include "MidiInput.h"
#include "MidiDefs.h"
#include "system.h"
#include "dspicsched.h"
#include "schedfuncs.h"
#include "PWMAudio.h"
#include "midihandler.h"
#include "math.h"

extern void Timer3ISR(void);
extern void _ISR _PWMInterrupt(void);


//Macros for Configuration Fuse Registers:
//Invoke macros to set up  device configuration fuse registers.
//The fuses will select the oscillator source, power-up timers, watch-dog
//timers, BOR characteristics etc. The macros are defined within the device
//header files. The configuration fuse registers reside in Flash memory.
_FOSC(CSW_FSCM_OFF & XT_PLL16);  //Run this project using an external crystal
                                //routed via the PLL in 16x multiplier mode
                                //For the 7.3728 MHz crystal we will derive a
                                //throughput of 7.3728e+6*16/4 = 29.4 MIPS(Fcy)
                                //,~33.9 nanoseconds instruction cycle time(Tcy).
_FWDT(WDT_OFF);                 //Turn off the Watch-Dog Timer.
_FBORPOR(MCLR_EN & PWRT_OFF& BORV_27 & PWRT_64 & MCLR_EN & PWMxL_ACT_HI & PWMxH_ACT_LO);   //Enable MCLR reset pin and turn off the
                                //power-up timers.
_FGS(CODE_PROT_OFF);            //Disable Code Protection


//void initParams(void);

void checkQ(float Q)
{
	if ((Q>1.0f)||(Q<-1.0f))
	{
		Q=0;
		while(1)
			;
	}
}

int main(void)
{
	{
/*
		BPF:        H(s) = (s/Q) / (s^2 + s/Q + 1)      (constant 0 dB peak gain)

            b0 =   alpha
            b1 =   0
            b2 =  -alpha
            a0 =   1 + alpha
            a1 =  -2*cos(w0)
            a2 =   1 - alpha


    w0 = 2*pi*f0/Fs

    cos(w0)
    sin(w0)

    alpha = sin(w0)/(2*Q)                                       (case: Q)
          = sin(w0)*sinh( ln(2)/2 * BW * w0/sin(w0) )           (case: BW)
          = sin(w0)/2 * sqrt( (A + 1/A)*(1/S - 1) + 2 )         (case: S)

*/
		const float w0 = 0.40f;
		float alpha;
		float cosW0;
		float Q = 12.0f;
		float a0;
		alpha = sin(w0)/(2*Q);
		cosW0 = cos(w0);
		a0 = (1.0f + alpha)*2.0f;

		Q = 2*cosW0 / a0;
	checkQ(Q);
		BIQ_coef_a1_a0 = Q15(Q);
		Q = -(1-alpha)/ a0;
	checkQ(Q);
		BIQ_coef_a2_a0 = Q15(Q);
		Q = alpha/ a0;
	checkQ(Q);
		BIQ_coef_b0_a0 = Q15(Q);
		BIQ_coef_b1_a0 = Q15(0);
		Q = 	-alpha / a0;
	checkQ(Q);
		BIQ_coef_b2_a0 = Q15(Q);
	}

	llist_init(&a);
	MISetupSerial();
	not_main();
	initParams();
	InitTimer123();
// INTCON1bits.NSTDIS = 1;

	llist_insert(&a, 100, &fun_metro,0);

	BIQ1_xn_0 = Q15(0.0f);
	BIQ1_xn_1 = Q15(0.0f);
	BIQ1_xn_2 = Q15(0.0f);


	while(1)
	{
		MIUartRXPollHandler();
		if (a.head)
		{
			if ((int)(a.head->time- time)>0)
			{
				TRISDbits.TRISD1 = 0;
				LATDbits.LATD1 = ! LATDbits.LATD1;
//				llist_pullhead(&a);
			}
		}
	}
}

