#include "midihandler.h"
#include "MidiInput.h"
#include "MidiDefs.h"
#include "PWMAudio.h"
#include "dspicsched.h"
#include "schedfuncs.h"
#include "math.h"

volatile char lastNote;
volatile int pitchBend;
volatile char portamento;
volatile char portamentoTime;

unsigned char w0 = 10;
unsigned char Q = 10;
	float alpha;
	float cosW0;
	float a0;
	float QQ;

void calcFilter()
{
	alpha = sin(.003f*w0)/(1.6f*Q);
	cosW0 = cos(.003f*w0);
	a0 = 0.5f/(1.0f + alpha);
	
	QQ = 2*cosW0 * a0;
	//	checkQ(QQ);
	BIQ_coef_a1_a0 = Q15(QQ);
	QQ = -(1-alpha)* a0;
	//	checkQ(QQ);
	BIQ_coef_a2_a0 = Q15(QQ);
	QQ = alpha * a0;
	//	checkQQ(Q);
	BIQ_coef_b0_a0 = Q15(QQ);
	BIQ_coef_b1_a0 = Q15(0);
	QQ = 	-alpha * a0;
	//	checkQ(Q);
	BIQ_coef_b2_a0 = Q15(QQ);

}

void calcPitch(void)
{
	int dp = __builtin_mulsu(pitchBend,341)>>14;
	inPitch1 = lastNote * 341 + 0x3000 + dp;
}


void MidiInNoteOn(void)
{
	if (!MidiByte2)
		MidiInNoteOff();
	else
	{	
		char ch;
		ch = MidiByte0 & 0x0F;
		if (ch == MidiInChannel)
		{
//			inPitch1 = MidiByte1 * 341 + 0x3000;
			char lnc;
			lnc = lastNote;
			lastNote = MidiByte1;
			calcPitch();
			if ((portamento == 0)||((portamento == 1)&&(lnc == 0)))
			{
				int *p;
				p = &outPitch1;
				*p++ = inPitch1;
				*p = inPitch1;
			}
			coefAmp1 = Env1Attack;
			Env1State = 1;
//			Env1Counter = Env1Hold;
			inAmp1 = MidiByte2*0x0100;
			llist_insert(&a, time + 1000, &fun_env,0);
		}
	}
}

void MidiInNoteOff(void)
{
	char ch = MidiByte0 & 0x0F;
	if (ch == MidiInChannel)
	{
		if (lastNote == MidiByte1)
		{
			Env1State = 2;
//			Env1Counter = 0xFFFF;	
			inAmp1 = 0;
			coefAmp1 = Env1Release;
			lastNote = 0;
		}
	}
}

void initParams(void)
{
	MidiByte0 =  MidiInChannel;
	lastNote = 0;

	MidiByte1 = MIDI_C_PORTA;
	MidiByte2 = 0;
	MidiInControlChange();

	MidiByte1 = MIDI_C_PORTA_TIME;
	MidiByte2 = 64;
	MidiInControlChange();

	MidiByte1 = MIDI_C_MODULATION;
	MidiByte2 = 10;
	MidiInControlChange();

	MidiByte1 = MIDI_C_CHORUS_DEPTH;
	MidiByte2 = 10;
	MidiInControlChange();

	MidiByte1 = MIDI_C_TREMELO_DEPTH;
	MidiByte2 = 30;
	MidiInControlChange();

	MidiByte1 = MIDI_C_GENERAL_1;
	MidiByte2 = 64;
	MidiInControlChange();

	MidiByte1 = MIDI_C_GENERAL_2;
	MidiByte2 = 64;
	MidiInControlChange();

	MidiByte1 = MIDI_C_GENERAL_3;
	MidiByte2 = 64;
	MidiInControlChange();

	MidiByte1 = MIDI_C_GENERAL_4;
	MidiByte2 = 64;
	MidiInControlChange();

	MidiByte1 = MIDI_C_GENERAL_5;
	MidiByte2 = 64;
	MidiInControlChange();

	MidiByte1 = MIDI_C_TREMELO_DEPTH;
	MidiByte2 = 64;
	MidiInControlChange();

	MidiByte1 = MIDI_C_EFFECT_DEPTH;
	MidiByte2 = 64;
	MidiInControlChange();

	MidiByte1 = 54;
	MidiByte2 = 8;
	MidiInNoteOn();

}

void MidiInControlChange(void)
{
	char ch = MidiByte0 & 0x0F;
	if (ch == MidiInChannel)
	{
		int p;
		switch(MidiByte1)
		{
			case MIDI_C_PORTA:
				portamento = MidiByte2; // 0 = off, 1 = auto, 2 = on
				break;
			case MIDI_C_PORTA_TIME: // scaling: every 32 units up is an octave up
				MidiByte2 = 128 - MidiByte2;
				coefPitch1 = ((0x20 + (MidiByte2 & 0x1F))<<(MidiByte2>>5)) - 0x1F;
				break;
			case MIDI_C_MODULATION:
				LFO2Amp = MidiByte2 << 3;
				break;	
			case MIDI_C_CHORUS_DEPTH:
				p = MidiByte2;
				LFO2Pitch =  (p<<7) + 0x0020;
				break;
			case MIDI_C_EFFECT_DEPTH: // LFO 1 -> amp modulation
				p = MidiByte2;
				LFO1Amp = (p<<8);
				break;
			case MIDI_C_TREMELO_DEPTH: // LFO 1 frequency
				p = MidiByte2;
//				LFO1Pitch =  (p<<7) + 0x0020;
				LFO1Pitch =  (p<<3) + 0x0001;
				break;
			case MIDI_C_GENERAL_1: 
				w0 = MidiByte2; calcFilter();
				MidiByte2 = 128 - MidiByte2;
				Env1Attack = ((0x40 + (MidiByte2 & 0x3F))<<(MidiByte2>>6)) - 0x3F;
				break;
			case MIDI_C_GENERAL_2:
				Env1Hold = ((0x10 + (MidiByte2 & 0x0F))<<(MidiByte2>>4)) * 0x10;
				Q = MidiByte2; calcFilter();
				break;
			case MIDI_C_GENERAL_3:
				MidiByte2 = 128 - MidiByte2;
				Env1Decay = ((0x40 + (MidiByte2 & 0x3F))<<(MidiByte2>>6)) - 0x3F;
				break;
			case MIDI_C_GENERAL_4:
				Env1Sustain = MidiByte2*0x0100;
				break;
			case MIDI_C_GENERAL_5:
				MidiByte2 = 128 - MidiByte2;
				Env1Release = ((0x40 + (MidiByte2 & 0x3F))<<(MidiByte2>>6)) - 0x3F;
				break;
			case MIDI_C_ALL_NOTES_OFF:
				{
					int *p;
					p = &outAmp1;
					*p++ = 0;
					*p = 0;
					Env1State = 2;
//					Env1Counter = 0xFFFF;	
					inAmp1 = 0;
					coefAmp1 = Env1Release;
					lastNote = 0;
				}
				break;
			default:
				break;
		}
	}
}

void MidiInPolyPressure(void)
{
}

void MidiInPitchBend(void)
{
	char ch = MidiByte0 & 0x0F;
	if (ch == MidiInChannel)
	{
		pitchBend = ((int)MidiByte2)<<7;
		pitchBend += MidiByte1;
//		pitchBend -= 0xE000;
		calcPitch();
	}
}

void MidiInSysexStart(void)
{
}

void MidiInMTC(void)
{
}

void MidiInSongPosition(void)
{
}

void MidiInSongSelect(void)
{
}

void MidiInTuneRequest(void)
{
}

void MidiInSysexEnd(void)
{
}

void MidiInTimingClock(void)
{
}

void MidiInMeasureEnd(void)
{
}

void MidiInStart(void)
{
}

void MidiInContinue(void)
{
}

void MidiInStop(void)
{
}

void MidiInActiveSense(void)
{
}

void MidiInReset(void)
{
}

void MidiInProgramChange(void)
{
}

void MidiInChannelPressure(void)
{
}
