; DSPMacros
;
; Macros:
;	PREPCOMPWAVETABLE1 TABLE
;	COMPWAVETABLE1 PHASORSTEP TABLE PHASOR OUTPUT
;	MTOF PITCH PHASORSTEP
;	MTOF2 PITCH PHASORSTEP
;	RNG16 SEED
;	LINEARINTERP INPUT TABLE OUTPUT
;	

;---------------------------------------------------------------------------
; macro PREPCOMPWAVETABLE1 
;
;	call this once before a series of COMPWAVETABLE calls
;	(p)reserve PSVPAG,XMODSRT,XMODEND, MODCON, W1...
;
; arguments: 
;	- PHASORSTEP: 32bit (linear freq) pitch
;	- TABLE : 256-word PSV (ROM) wavetable
;	
;	execution time : 9 cycles
;---------------------------------------------------------------------------
.macro PREPCOMPWAVETABLE1 TABLE
		mov 	#psvpage(&TABLE&),W0
		mov 	W0,PSVPAG
		mov		#psvoffset(&TABLE&),W0
		mov		W0,XMODSRT
		mov		#psvoffset(&TABLE&)+#0x1FF,W0
		mov		W0,XMODEND
		mov		#0x8008,W0; W8 is circular adressing ptr
		mov		W0,MODCON
		mov		#0x7FFF,W1
		mov		#psvoffset(&TABLE&),W3
.endm

;---------------------------------------------------------------------------
; macro COMPWAVETABLE1 
;
;	computes one sample of a wavetable synth
; 	linear interpolated 256pt wavetable
;
; 	todo: array in/out operation?
;
; 	phasor (and phasorstep) format:
;   --------MSB-------- --------LSB--------
;   xxxx xxxi iiii iiif ffff ffff ffff ffxx
; 	(i = index in table, f = interpolated fraction)
;
; arguments: 
;	- PHASORSTEP (r): 32bit (linear freq) pitch
;	- TABLE (ro): 256-word PSV (ROM) wavetable
;	- PHASOR (rw): 32bit (linear freq) pitch (keep private)
;	- OUTPUT (w): 16bit wave
;
;	context:
; 	- destroys WREGs, requires W1,W3 init from PREPCOMPWAVETABLE1
;	
;	execution time : 15 cyc
;---------------------------------------------------------------------------

.macro PREP2COMPWAVETABLE1 TABLE PHASORSTEP PHASOR
		mov		#&PHASORSTEP&,W0
		add	 	#&PHASOR&,
.endm

.macro COMPWAVETABLE1 PHASORSTEP TABLE PHASOR ;OUTPUT
;		bclr	update,#1
; Step the phasor
		mov		&PHASORSTEP&,W0
		add	 	&PHASOR&
		mov	 	&PHASORSTEP&+2,W0
		addc 	&PHASOR&+2
; linear interpolation...
		mov 	&PHASOR&+2,W8
		and 	#0x01FE,W8
		add		W8,W3,W8 
;		add	 	W8,#2,W8
		lsr		&PHASOR&+2,WREG
		rrc		&PHASOR&,WREG
		lsr		W0,#1,W5
		mov	 	[W8++], W4
		xor	 	W5,W1,W6		
		mpy		W4*W6,A,[W8],W4
		mac		W4*W5,A
		sac.r 	A, #0, [W7++]
;		mov		W0,&OUTPUT&
.endm

;---------------------------------------------------------------------------
; macro MTOF
;
; Midi to frequency 
; actually not frequency but wavetable pointer step
; actually not midi but 0xoffg where o = octave, ff = fraction of octave, g = unused
;
; precision: NON-interpolated: 256 steps per octave table = +-22 steps per semitone
;
;	arguments:
;	- PITCH (r) : unsigned 16 bit word 
;	- PHASORSTEP (w) : unsigned 32 bit word 
;
;	context:
; 	- destroys WREGs
;	
;	execution time : 18 cyc
;---------------------------------------------------------------------------
.macro MTOF PITCH PHASORSTEP
		mov 	#psvpage(ExpTable),W0
		mov 	W0,PSVPAG
		mov		#0x0FF0,W0
		and		&PITCH&,WREG
		lsr		W0,#3,W0
		mov		#psvoffset(ExpTable),W1
		add		W0,W1,W0
		mov		[W0],W1
		clr 	A
		mov		W1,ACCAL
		mov		&PITCH&,WREG
		lsr		W0,#12,W0
		subr	W0,#9,W0
		sftac	A,W0
		mov		ACCAH,W0
		mov		W0,&PHASORSTEP&+2
		mov		ACCAL,W0
		mov 	W0,	&PHASORSTEP&
.endm

;---------------------------------------------------------------------------
; macro MTOF2 
;
; Midi to frequency 
; actually not frequency but wavetable pointer step
; actually not midi but 0xoffg where o = octave, ff = fraction of octave, g = interpolated fraction
;
; precision: linear interpolated 256 steps per octave table
;
;	arguments:
;	- PITCH (r) : unsigned 16 bit word 
;	- PHASORSTEP (w) : unsigned 32 bit word 
;
;	context:
; 	- destroys WREGs
;	
;	execution time : 18n+2 cycles
;---------------------------------------------------------------------------
; 22*8 = 176
; 20*8+2 = 162
; 16*8+2 = 130
; 14*8+4 = 116

.macro PREPMTOF2
		mov		#psvoffset(ExpTable),W2
		mov		#0x0FF0,W3
		mov		#ACCAL,W10;
.endm

.macro PREP2MTOF2 PHASORSTEP
		mov		#&PHASORSTEP&,W7;
.endm

.macro MTOF2 PITCH 
		mov		&PITCH,W1
		and		W3,W1,W0
		lsr		W0,#3,W0
		add		W0,W2,W9
		and		W1,#0x0F,W5
		mov		[W9++],W4
		xor	 	W5,#0x0F,W6				
		mpy		W4*W6,A,[W9],W4
		mac		W4*W5,A
		lsr		W1,#12,W0
		subr	W0,#9,W0
		sftac	A,W0
		mov		[W10++],[W7++]
;		mov 	W0,	&PHASORSTEP&
		mov		[W10--],[W7++]
;		mov		W0,&PHASORSTEP&+2
 .endm


;---------------------------------------------------------------------------
; macro RNG16 seed
;
;	16 bit LFSR pseudo-random generator
;
;	arguments:
;	- seed (rw) : 16 bit seed input and new value output
;
;	context:
; 	- destroys SR
;---------------------------------------------------------------------------
.macro	RNG16 SEED
		bclr	SR,#C
		btss	&SEED&,#15
		btg		SR,#C
		btss	&SEED&,#14
		btg		SR,#C
		btss	&SEED&,#12
		btg		SR,#C
		btss	&SEED&,#3
		btg		SR,#C
		rlc		&SEED&
.endm


;---------------------------------------------------------------------------
; macro LINEARINTERP
;
; document...
;
;	arguments:
;	- INPUT (r): 32 bit 
; 		format:
;  		--------MSB-------- --------LSB--------
;  		xxxx xxxi iiii iiif ffff ffff ffff ffxx
; 		(x = don't care, i = integer part, f = fractional part)
;	- TABLE (r): PSV ROM TABLE
;	- OUTPUT: 16 bit
;---------------------------------------------------------------------------
.macro LINEARINTERP INPUT TABLE OUTPUT
		mov 	#psvpage(&TABLE&),W0
		mov 	W0,PSVPAG
; linear interpolation...
		mov 	&INPUT&+2,W8
		and 	#0x01FE,W8
		mov		#psvoffset(&TABLE&),W0
		add		W8,W0,W8 
		mov	 	[W8++], W4
;		add	 	W8,#2,W8
		lsr		&INPUT&+2,WREG
		rrc		&INPUT&,WREG
		lsr		W0,#1,W5
		mov		#0x7FFF,W0
		xor	 	W5,W0,W6		
		mpy		W4*W6,A,[W8],W4
		mac		W4*W5,A
		sac.r 	A, #0, W0
		mov		W0,&OUTPUT&
.endm


;---------------------------------------------------------------------------
; macro LPF1S
;
; First-order LPF
; Output has 32 bits precision for good convergence
; Inputs have 16 bits
; low COEF = slow convergence
; 
;	context:
;		- destroys W0,W1,W2
;	OK
;---------------------------------------------------------------------------

.macro LPF1S INPUT COEF OUTPUT
		mov		&INPUT,W0
		subr	&OUTPUT,WREG
		mov		&COEF,W1
		mul.su	W0,W1,W0
		add		&OUTPUT+2
		mov		W1,W0
		addc	&OUTPUT
.endm


;---------------------------------------------------------------------------
; macro WORDFILLER
;
;	applies key-value pairs, starts reading at W1
;
;	context:
;		- destorys W1,W2
;	KVPTR result is in W1
;	OK
;---------------------------------------------------------------------------
.macro 	WORDFILLER
		mov		[W1++],W0
		add		W0,#0,W0
		bra		Z,$+6
		mov		[W1++],[W0]
		bra 	$-8
.endm



;---------------------------------------------------------------------------
; macro RNDWORDFILLER
;
;	applies key-value pairs with random values
;	[adress],[randomRange],[bias]
;	adress = bias + rand()%randomRang
;	starts reading at W1
;
;	arguments:
;		- LBL: unique label
;	context:
;		- destroys W1,W2,W3,W4
;	KVPTR result is in W1
;---------------------------------------------------------------------------
.macro 	RNDWORDFILLER LBL
		mov		W1,W3
&LBL:
		RNG16 	rng1
		mov		[W3++],W2
		add		W2,#0,W2
		bra		Z,$+14
		mov		[W3++],W0
		mov		rng1,W1
		mul.uu	W0,W1,W0
		mov		[W3++],W0
		add		W1,W0,[W2]
		bra 	&LBL
		mov		W3,W1
.endm

;---------------------------------------------------------------------------
; macro CALLCENTER
;---------------------------------------------------------------------------
.macro CALLCENTER LBL
A&LBL:
		mov		[W1++],W0
		cp0		W0
		bra		Z,B&LBL
		call	W0	;	W1 is reserved here
		bra		A&LBL
B&LBL:

.endm
;---------------------------------------------------------------------------
; macro RNDSTEP
;---------------------------------------------------------------------------
.macro RNDSTEP LBL
		mov		[W1--],W2
		mov		rng1,W0
		lsr		W0,#1,W0
		dec		WREG0
&LBL:
		mov		[W1++],W2
		mov		[W1++],W2
		cpslt	W0,W2
		bra		&LBL
		mov		[W1],W0
		mov		W0,W1
.endm

;---------------------------------------------------------------------------
; macro SUMOFPRODUCTS
;
;	arguments:
;	- X (r): signed 16 bit array
;	- Y (r): signed 16 bit array
;	- N 
