; DSPMacros2 has vector operations for music-dsp
; by Johannes Taelman
; started on 20061030
		.include "DSPMacros.s"

;---------------------------------------------------------------------------
; macro LPF1S_B8
;
; First-order LPF
; Output has 16 bits precision
; 	needs 32 bit object storage - for good convergence
; Inputs have 16 bits
; low COEF = slow convergence
;
;
; 
;	context:
;		- destroys W0,W1,W2,W4,W5,W7,Acc
;	OK
;---------------------------------------------------------------------------
; performance before : 7N => 56 cycles for 8 samples
; performance after:  4N + 14 = 46 cycles for 8 samples hmmm

; performance before : 7N => 112 cycles for 16 samples
; performance after:  4N + 14 = 78 cycles for 16 samples hmmm


.macro LPF1S_B8 INPUT COEF OUTPUT TEMP32
;		mov		&INPUT,W0		; W0 = input
;		subr	&OUTPUT,WREG	; W0 = input - output
;		mov		&COEF,W1		; W1 = coef
;		mul.su	W0,W1,W0		; W01 = coef * (input - output)
;		add		&OUTPUT+2		; Output32 = Output32+ coef * (input - output)
;		mov		W1,W0
;		addc	&OUTPUT

		clr 	A
		mov		&TEMP32&,W0;
		mov		W0,ACCAL
		mov		&TEMP32&+2,W0;
		mov 	W0,ACCAH
		
		mov  	#&INPUT&,W0
		mov  	#&OUTPUT&,W1
		mov 	&COEF,W7
		asr		W7,#1,W7 ;?
		mov		ACCAH,W4

		DO #8, LPF1S_B8_l_end
				subr	W4,[W0++],W5
				mac		W5*W7,A
				mov		ACCAH,W4
LPF1S_B8_l_end:	sac		a,[W1++]
;	;mov		W4,[W1++]

		mov		ACCAL,W0;
		mov		W0,&TEMP32
		mov		ACCAH,W0;
		mov		W0,&TEMP32+2
.endm

				
;---------------------------------------------------------------------------
; macro K_TO_A_SMOOTH_B8
;
; First-order LPF after sample&hold upsampling from K-rate to A-rate
; Output has 16 bits precision
; 	needs 32 bit object storage - for good convergence
; Inputs have 16 bits
; low COEF = slow convergence
;
;
; 
;	context:
;		- destroys W0,W1,W2,W4,W5,W7,Acc
;	OK
;---------------------------------------------------------------------------
; performance before : 7N => 56 cycles for 8 samples
; performance after:  2N + 16 = 32 cycles for 8 samples 

; performance before : 7N => 112 cycles for 16 samples
; performance after:  2N + 16 = 48 cycles for 16 samples hmmm


.macro K_TO_A_SMOOTH_B8 INPUT COEF OUTPUT TEMP32
;		mov		&INPUT,W0		; W0 = input
;		subr	&OUTPUT,WREG	; W0 = input - output
;		mov		&COEF,W1		; W1 = coef
;		mul.su	W0,W1,W0		; W01 = coef * (input - output)
;		add		&OUTPUT+2		; Output32 = Output32+ coef * (input - output)
;		mov		W1,W0
;		addc	&OUTPUT

		clr 	A
		mov		&TEMP32&,W0;
		mov		W0,ACCAL
		mov		&TEMP32&+2,W0;
		mov 	W0,ACCAH
		
		mov  	#&INPUT,W0
		mov  	#&OUTPUT,W1
		mov 	&COEF,W7
		asr		W7,#1,W7 ;?
		mov		ACCAH,W4
		subr	W4,[W0],W5

		DO #7, LPF1S_B8_l_end&TEMP32
				mac		W5*W7,A
				mov		ACCAH,W0
LPF1S_B8_l_end&TEMP32:	mov W0,[W1++]

		mov		ACCAL,W0;
		mov		W0,&TEMP32
		mov		ACCAH,W0;
		mov		W0,&TEMP32+2
.endm


.macro MTOF2_B8 PITCH1 PHASORSTEP1
	PREPMTOF2
	PREP2MTOF2 \PHASORSTEP1
	MTOF2 \PITCH1+0 
	MTOF2 \PITCH1+2 
	MTOF2 \PITCH1+4 
	MTOF2 \PITCH1+6 
	MTOF2 \PITCH1+8 
	MTOF2 \PITCH1+10
	MTOF2 \PITCH1+12
	MTOF2 \PITCH1+14
.endm


;---------------------------------------------------------------------------
; macro COMPWAVETABLE1 
;
;	computes one sample of a wavetable synth
; 	linear interpolated 256pt wavetable
;
; 	todo: array in/out operation?
;
; 	phasor (and phasorstep) format:
;   --------MSB-------- --------LSB--------
;   xxxx xxxi iiii iiif ffff ffff ffff ffxx
; 	(i = index in table, f = interpolated fraction)
;
; arguments: 
;	- PHASORSTEP (r): 32bit (linear freq) pitch
;	- TABLE (ro): 256-word PSV (ROM) wavetable
;	- PHASOR (rw): 32bit (linear freq) pitch (keep private)
;	- OUTPUT (w): 16bit wave
;
;	context:
; 	- destroys WREGs
;	
;	execution time : 17 cyc
;---------------------------------------------------------------------------

; phasor in accA
; phasorstep in accB

.macro _COMPWAVETABLE1 PHASORSTEP TABLE PHASOR OUTPUT
;		bclr	update,#1
; Step the phasor
		mov		&PHASORSTEP&,W0
		add	 	&PHASOR&
		mov	 	&PHASORSTEP&+2,W0
		addc 	&PHASOR&+2
; linear interpolation...
		mov 	&PHASOR&+2,W8
		and 	#0x01FE,W8
		mov		#psvoffset(&TABLE&),W0
		add		W8,W0,W8 
		mov	 	[W8++], W4
;		add	 	W8,#2,W8
		lsr		&PHASOR&+2,WREG
		rrc		&PHASOR&,WREG
		lsr		W0,#1,W5
		xor	 	W5,W1,W6		
		mpy		W4*W6,A,[W8],W4
		mac		W4*W5,A
		sac.r 	A, #0, W0
		mov		W0,&OUTPUT&
.endm

.macro COMPWAVETABLE1_B8 PHASORSTEP1 TABLE1 PHASOR1 OUTPUT1
	PREPCOMPWAVETABLE1 \TABLE1
	mov #\OUTPUT1,W7
	COMPWAVETABLE1 \PHASORSTEP1 \TABLE1 \PHASOR1 ;\OUTPUT1
	COMPWAVETABLE1 \PHASORSTEP1+4 \TABLE1 \PHASOR1 ;\OUTPUT1+2
	COMPWAVETABLE1 \PHASORSTEP1+8 \TABLE1 \PHASOR1 ;\OUTPUT1+4
	COMPWAVETABLE1 \PHASORSTEP1+12 \TABLE1 \PHASOR1 ;\OUTPUT1+6
	COMPWAVETABLE1 \PHASORSTEP1+16 \TABLE1 \PHASOR1 ;\OUTPUT1+8
	COMPWAVETABLE1 \PHASORSTEP1+20 \TABLE1 \PHASOR1 ;\OUTPUT1+10
	COMPWAVETABLE1 \PHASORSTEP1+24 \TABLE1 \PHASOR1 ;\OUTPUT1+12
	COMPWAVETABLE1 \PHASORSTEP1+28 \TABLE1 \PHASOR1 ;\OUTPUT1+14
.endm

.macro COMPWAVETABLE1_B8_C  PHASORSTEP1 TABLE1 PHASOR1 OUTPUT1
	PREPCOMPWAVETABLE1 \TABLE1
	mov #\OUTPUT1,W7
	COMPWAVETABLE1 \PHASORSTEP1 \TABLE1 \PHASOR1 ;\OUTPUT1
	COMPWAVETABLE1 \PHASORSTEP1 \TABLE1 \PHASOR1 ;\OUTPUT1+2
	COMPWAVETABLE1 \PHASORSTEP1 \TABLE1 \PHASOR1 ;\OUTPUT1+4
	COMPWAVETABLE1 \PHASORSTEP1 \TABLE1 \PHASOR1 ;\OUTPUT1+6
	COMPWAVETABLE1 \PHASORSTEP1 \TABLE1 \PHASOR1 ;\OUTPUT1+8
	COMPWAVETABLE1 \PHASORSTEP1 \TABLE1 \PHASOR1 ;\OUTPUT1+10
	COMPWAVETABLE1 \PHASORSTEP1 \TABLE1 \PHASOR1 ;\OUTPUT1+12
	COMPWAVETABLE1 \PHASORSTEP1 \TABLE1 \PHASOR1 ;\OUTPUT1+14
.endm
