#define MidiInChannel 0

#include "MidiInput.h"
#include "MidiDefs.h"
#include "system.h"



extern void Timer3ISR(void);
extern void PulseHoldInit(void);
//extern void PulseHoldNoteOn(char n, char m);
//extern void PulseHoldNoteOff(char n);
extern void PulseHoldStart(char n, char velo);
extern void HoldStop(char TimerNumber);

extern void not_main(void);
extern void test1(void);

extern int inPitch1;
extern volatile int outPitch1;
extern volatile int outAmp1;
extern int inAmp1;
extern int coefPitch1;
extern int coefAmp1;
extern int LFO1Amp;
extern int LFO2Amp;
extern int LFO1Pitch;
extern int LFO2Pitch;
extern int Env1State;
extern int Env1Attack;
extern int Env1Decay;
extern int Env1Sustain;
extern int Env1Release;
extern int Env1Counter;
extern int Env1Hold;

//Macros for Configuration Fuse Registers:
//Invoke macros to set up  device configuration fuse registers.
//The fuses will select the oscillator source, power-up timers, watch-dog
//timers, BOR characteristics etc. The macros are defined within the device
//header files. The configuration fuse registers reside in Flash memory.
_FOSC(CSW_FSCM_OFF & XT_PLL16);  //Run this project using an external crystal
                                //routed via the PLL in 16x multiplier mode
                                //For the 7.3728 MHz crystal we will derive a
                                //throughput of 7.3728e+6*16/4 = 29.4 MIPS(Fcy)
                                //,~33.9 nanoseconds instruction cycle time(Tcy).
_FWDT(WDT_OFF);                 //Turn off the Watch-Dog Timer.
_FBORPOR(MCLR_EN & PWRT_OFF& BORV_27 & PWRT_64 & MCLR_EN & PWMxL_ACT_HI & PWMxH_ACT_LO);   //Enable MCLR reset pin and turn off the
                                //power-up timers.
_FGS(CODE_PROT_OFF);            //Disable Code Protection

volatile char lastNote;
volatile int pitchBend;
volatile char portamento;
volatile char portamentoTime;

void initParams(void);

int main(void)
{
	int s;
	char a;

//	test1();

	MISetupSerial();
	not_main();
	initParams();

	T2CONbits.TON = 1;
	T2CONbits.T32 = 1;

	s = 0;

	while(1)
	{
		MIUartRXPollHandler();

	}
}

void calcPitch(void)
{
	int dp = __builtin_mulsu(pitchBend,341)>>14;
	inPitch1 = lastNote * 341 + 0x3000 + dp;
}

void MidiInNoteOn(void)
{
	if (!MidiByte2)
		MidiInNoteOff();
	else
	{	
		char ch;
		ch = MidiByte0 & 0x0F;
		if (ch == MidiInChannel)
		{
//			inPitch1 = MidiByte1 * 341 + 0x3000;
			char lnc;
			lnc = lastNote;
			lastNote = MidiByte1;
			calcPitch();
			if ((portamento == 0)||((portamento == 1)&&(lnc == 0)))
			{
				int *p;
				p = &outPitch1;
				*p++ = inPitch1;
				*p = inPitch1;
			}
			coefAmp1 = Env1Attack;
			Env1State = 0;
			Env1Counter = Env1Hold;
			inAmp1 = MidiByte2*0x0100;
		}
	}
}

void MidiInNoteOff(void)
{
	char ch = MidiByte0 & 0x0F;
	if (ch == MidiInChannel)
	{
		if (lastNote == MidiByte1)
		{
			Env1State = 2;
			Env1Counter = 0xFFFF;	
			inAmp1 = 0;
			coefAmp1 = Env1Release;
			lastNote = 0;
		}
	}
}

void initParams(void)
{
	MidiByte0 =  MidiInChannel;
	lastNote = 0;

	MidiByte1 = MIDI_C_PORTA;
	MidiByte2 = 0;
	MidiInControlChange();

	MidiByte1 = MIDI_C_PORTA_TIME;
	MidiByte2 = 64;
	MidiInControlChange();

	MidiByte1 = MIDI_C_MODULATION;
	MidiByte2 = 10;
	MidiInControlChange();

	MidiByte1 = MIDI_C_CHORUS_DEPTH;
	MidiByte2 = 20;
	MidiInControlChange();

	MidiByte1 = MIDI_C_TREMELO_DEPTH;
	MidiByte2 = 30;
	MidiInControlChange();

	MidiByte1 = MIDI_C_GENERAL_1;
	MidiByte2 = 64;
	MidiInControlChange();

	MidiByte1 = MIDI_C_GENERAL_2;
	MidiByte2 = 64;
	MidiInControlChange();

	MidiByte1 = MIDI_C_GENERAL_3;
	MidiByte2 = 64;
	MidiInControlChange();

	MidiByte1 = MIDI_C_GENERAL_4;
	MidiByte2 = 64;
	MidiInControlChange();

	MidiByte1 = MIDI_C_GENERAL_5;
	MidiByte2 = 64;
	MidiInControlChange();

	MidiByte1 = MIDI_C_TREMELO_DEPTH;
	MidiByte2 = 64;
	MidiInControlChange();

	MidiByte1 = MIDI_C_EFFECT_DEPTH;
	MidiByte2 = 64;
	MidiInControlChange();

	MidiByte1 = 64;
	MidiByte2 = 64;
	MidiInNoteOn();

}

void MidiInControlChange(void)
{
	char ch = MidiByte0 & 0x0F;
	if (ch == MidiInChannel)
	{
		int p;
		switch(MidiByte1)
		{
			case MIDI_C_PORTA:
				portamento = MidiByte2; // 0 = off, 1 = auto, 2 = on
				break;
			case MIDI_C_PORTA_TIME: // scaling: every 32 units up is an octave up
				MidiByte2 = 128 - MidiByte2;
				coefPitch1 = ((0x20 + (MidiByte2 & 0x1F))<<(MidiByte2>>5)) - 0x1F;
				break;
			case MIDI_C_MODULATION:
				LFO2Amp = MidiByte2 << 3;
				break;	
			case MIDI_C_CHORUS_DEPTH:
				p = MidiByte2;
				LFO2Pitch =  (p<<7) + 0x0020;
				break;
			case MIDI_C_EFFECT_DEPTH: // LFO 1 -> amp modulation
				p = MidiByte2;
				LFO1Amp = (p<<8);
				break;
			case MIDI_C_TREMELO_DEPTH: // LFO 1 frequency
				p = MidiByte2;
				LFO1Pitch =  (p<<7) + 0x0020;
				break;
			case MIDI_C_GENERAL_1: 
				MidiByte2 = 128 - MidiByte2;
				Env1Attack = ((0x40 + (MidiByte2 & 0x3F))<<(MidiByte2>>6)) - 0x3F;
				break;
			case MIDI_C_GENERAL_2:
				Env1Hold = ((0x10 + (MidiByte2 & 0x0F))<<(MidiByte2>>4)) * 0x10;
				break;
			case MIDI_C_GENERAL_3:
				MidiByte2 = 128 - MidiByte2;
				Env1Decay = ((0x40 + (MidiByte2 & 0x3F))<<(MidiByte2>>6)) - 0x3F;
				break;
			case MIDI_C_GENERAL_4:
				Env1Sustain = MidiByte2*0x0100;
				break;
			case MIDI_C_GENERAL_5:
				MidiByte2 = 128 - MidiByte2;
				Env1Release = ((0x40 + (MidiByte2 & 0x3F))<<(MidiByte2>>6)) - 0x3F;
				break;
			case MIDI_C_ALL_NOTES_OFF:
				{
					int *p;
					p = &outAmp1;
					*p++ = 0;
					*p = 0;
					Env1State = 2;
					Env1Counter = 0xFFFF;	
					inAmp1 = 0;
					coefAmp1 = Env1Release;
					lastNote = 0;
				}
				break;
			default:
				break;
		}
	}
}

void MidiInPolyPressure(void)
{
}

void MidiInPitchBend(void)
{
	char ch = MidiByte0 & 0x0F;
	if (ch == MidiInChannel)
	{
		pitchBend = ((int)MidiByte2)<<7;
		pitchBend += MidiByte1;
//		pitchBend -= 0xE000;
		calcPitch();
	}
}

void MidiInSysexStart(void)
{
}

void MidiInMTC(void)
{
}

void MidiInSongPosition(void)
{
}

void MidiInSongSelect(void)
{
}

void MidiInTuneRequest(void)
{
}

void MidiInSysexEnd(void)
{
}

void MidiInTimingClock(void)
{
}

void MidiInMeasureEnd(void)
{
}

void MidiInStart(void)
{
}

void MidiInContinue(void)
{
}

void MidiInStop(void)
{
}

void MidiInActiveSense(void)
{
}

void MidiInReset(void)
{
}

void MidiInProgramChange(void)
{
}

void MidiInChannelPressure(void)
{
}


extern void __attribute__((__interrupt__)) _PWMInterrupt(void);
