#include "MidiInput.h"
#include "MidiDefs.h"
#include "System.h"

const unsigned char StatusLengthLookup[16] = 
{
 0,0,0,0,0,0,0,0,
 3, // 0x80=note off, 3 bytes
 3, // 0x90=note on, 3 bytes
 3, // 0xa0=poly pressure, 3 bytes
 3, // 0xb0=control change, 3 bytes
 2, // 0xc0=program change, 2 bytes
 2, // 0xd0=channel pressure, 2 bytes
 3, // 0xe0=pitch bend, 3 bytes
 -1 // 0xf0=other things. may vary.
};

const signed char SysMsgLengthLookup[16] =
{
 -1, // 0xf0=sysex start. may vary
 2, // 0xf1=MIDI Time Code. 2 bytes
 3, // 0xf2=MIDI Song position. 3 bytes
 2, // 0xf3=MIDI Song Select. 2 bytes.
 0, // 0xf4=undefined
 0, // 0xf5=undefined
 1, // 0xf6=TUNE Request
 0, // 0xf7=sysex end.
 1, // 0xf8=timing clock. 1 byte
 1, // 0xf9=proposed measure end?
 1, // 0xfa=start. 1 byte
 1, // 0xfb=continue. 1 byte
 1, // 0xfc=stop. 1 byte
 0, // 0xfd=undefined
 1, // 0xfe=active sensing. 1 byte
 3 // 0xff= not reset, but a META-EVENT, which is always 3 bytes
};

/*
					switch (MidiByte0)
					{
						case MIDI_SYSEX_START:
							MidiInSysexStart();
							break;
						case MIDI_MTC:
							MidiInMTC();
							break;
						case MIDI_SONG_POSITION:
							MidiInSongPosition();
							break;
						case MIDI_SONG_SELECT:
							MidiInSongSelect();
							break;		
						case MIDI_SYSEX_END:
							MidiInSysexEnd();
							break;	
						case MIDI_RESET:
							MidiInReset();
							break;
					}
*/


#define RXBUFLENGTH 16 // must be a power of two
#define TXBUFLENGTH 16 // must be a power of two
#define SPBRG_VAL 0x4F

unsigned char RXBuf[RXBUFLENGTH];
unsigned char RXBufWRIndex;
unsigned char RXBufRDIndex;
unsigned char TXBuf[TXBUFLENGTH];
unsigned char TXBufWRIndex;
unsigned char TXBufRDIndex;
unsigned char MidiByte0;
unsigned char MidiByte1;
unsigned char MidiByte2;
unsigned char MidiCurData;
unsigned char MidiNumData;
extern unsigned char MidiInChannel;


#define BAUDRATE        31250                   //Desired Baud Rate
#define BRGVAL          ((FCY/BAUDRATE)/16)-1   //Formula for U1BRG register
                                                //from dsPIC30F Family
                                                //Reference Manual


#pragma code


void MISetupSerial(void)
{
	RXBufWRIndex = 0;
	RXBufRDIndex = 0;

	U1MODE = 0x0000;        //Clear UART2 registers
	U1STA = 0x0000;
	//Since the SPI1 (SCK1, SDO1, SDI1) pins are multiplexed with the UART
	//(U1TX and U1RX ) pins on this device this demonstration program
	//will use alternate UART1 pins (U1ATX and U1ARX)
	//The SPI1 module is used to comunicate to the LCD Controller while
	//UART1 module is used to communicate with the RS232 port on the PC
	U1MODEbits.ALTIO = 1;   //Enable U1ATX and U1ARX instead of
	                      //U1TX and U1RX pins
	
	U1MODEbits.UARTEN = 1;  //Enable UART1 module
	U1BRG = BRGVAL;         //Load UART1 Baud Rate Generator
	
	IFS0bits.U1RXIF = 0;    //Clear UART1 Receiver Interrupt Flag
	IFS0bits.U1TXIF = 0;    //Clear UART1 Transmitter Interrupt Flag
	IEC0bits.U1RXIE = 1;    //Enable UART1 Receiver ISR
	IEC0bits.U1TXIE = 1;    //Enable UART1 Transmitter ISR
	U1STAbits.UTXISEL = 0;  //Setup UART1 transmitter to interrupt
	                      //when a character is transferred to the
	                      //Transmit Shift register and as result,
	                      //the transmit buffer becomes empty.
	
	U1STAbits.UTXEN = 1;    //Enable UART1 transmitter
}


void MIUartRXPollHandler(void)
{
	if (RXBufWRIndex - RXBufRDIndex)
	{
		unsigned char data,rdptr;
		char len;
		rdptr = RXBufRDIndex+1;
		rdptr &= (RXBUFLENGTH-1);
		RXBufRDIndex = rdptr;
		data = RXBuf[RXBufRDIndex];
		if (data & 0x80)
		{
			MidiByte0 = data;
			len = StatusLengthLookup[MidiByte0>>4];
			if (len == -1)
			{
				len = SysMsgLengthLookup[MidiByte0-0xF0];
				if (len == 1)
				{
					switch (MidiByte0)
					{
						case MIDI_TUNE_REQUEST:
							MidiInTuneRequest();
							break;	
						case MIDI_TIMING_CLOCK:
							MidiInTimingClock();
							break;	
						case MIDI_MEASURE_END:
							MidiInMeasureEnd();
							break;	
						case MIDI_START:
							MidiInStart();
							break;	
						case MIDI_CONTINUE:
							MidiInContinue();
							break;	
						case MIDI_STOP:
							MidiInStop();
							break;	
						case MIDI_ACTIVE_SENSE:
							MidiInActiveSense();
							break;	
						default :
							break;
					}
				}
				else
				{
					MidiNumData = len-1;
					MidiCurData = 0;
				}
			}
			else
			{
				MidiNumData = len-1;
				MidiCurData = 0;
			}
		}
		else // not a status byte
		{
			if (MidiCurData == 0)
			{
				MidiByte1 = data;
				if (MidiNumData == 1)
				{
					// 2 byte message complete
					switch (MidiByte0 && 0xF0)
					{
						case MIDI_PROGRAM_CHANGE:
							MidiInProgramChange();
							break;
						case MIDI_CHANNEL_PRESSURE:
							MidiInChannelPressure();
							break;							
						default :
						{
						}
					}					
					MidiCurData = 0;
				}
				else
					MidiCurData++;
			}
			else if (MidiCurData == 1)
			{
				MidiByte2 = data;
				if (MidiNumData == 2)
				{
					// 3 byte message complete
					switch (MidiByte0 & 0xF0)
					{
						case MIDI_NOTE_ON:
							MidiInNoteOn();
							break;
						case MIDI_NOTE_OFF:
							MidiInNoteOff();
							break;
						case MIDI_POLY_PRESSURE:
							MidiInPolyPressure();
							break;
						case MIDI_CONTROL_CHANGE:
							MidiInControlChange();
							break;
						case MIDI_PITCH_BEND:
							MidiInPitchBend();
							break;
						default:
						{}
						break;
					}
					MidiCurData = 0;
				}
			}			
		}
	}
}


//_U2TXInterrupt() is the UART2 Interrupt Service Routine.
//The routine must have global scope in order to be an ISR.
//The ISR name is the same name provided for the module in the device linker
//script.
//The UART2 ISR loads the UART2 4-deep FIFO buffers with the next
//4 characters in the Display buffer unless it encounters a null character.
void __attribute__((__interrupt__)) _U1TXInterrupt(void)
{
	if (TXBufWRIndex - TXBufRDIndex)
	{
		unsigned char data;
		TXBufRDIndex++;
		TXBufRDIndex&=(TXBUFLENGTH-1);
		U1TXREG = TXBuf[TXBufRDIndex];
	if (TXBufWRIndex - TXBufRDIndex)
	{
		unsigned char data;
		TXBufRDIndex++;
		TXBufRDIndex&=(TXBUFLENGTH-1);
		U1TXREG = TXBuf[TXBufRDIndex];
	if (TXBufWRIndex - TXBufRDIndex)
	{
		unsigned char data;
		TXBufRDIndex++;
		TXBufRDIndex&=(TXBUFLENGTH-1);
		U1TXREG = TXBuf[TXBufRDIndex];
	}
	}
	}
    IFS0bits.U1TXIF = 0;    //Clear the UART1 transmitter interrupt flag
}


void __attribute__((__interrupt__)) _U1RXInterrupt(void)
{
	IFS0bits.U1RXIF = 0;    //Clear UART1 Receiver Interrupt Flag
	unsigned char ch = RXBufWRIndex;
	while (U1STAbits.URXDA)
	{
		ch=(ch+1)&(RXBUFLENGTH-1);	
		RXBuf[ch] = U1RXREG;
	}
	RXBufWRIndex = ch;
}
